package com.agiledeveloper;

import clojure.lang.LockingTransaction;
import clojure.lang.Ref;

import java.util.concurrent.Callable;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

public class EnergySource {
  private final long MAXLEVEL = 100;
  private Ref level = new Ref(MAXLEVEL);
  private Ref usageCount = new Ref(0L);

  private static ScheduledExecutorService timer =
      Executors.newScheduledThreadPool(10);

  private EnergySource() {
  }

  private void init() {
    timer.scheduleAtFixedRate(new Runnable() {
      public void run() {
        replenish();
      }
    }, 0, 1, TimeUnit.SECONDS);
  }

  public static void stopAllEnergySources() {
    timer.shutdown();
  }

  public static EnergySource create() {
    EnergySource energySource = new EnergySource();
    energySource.init();
    return energySource;
  }

  public long getUnitsAvailable() {
    return (Long) level.deref();
  }

  public long getUsage() {
    return (Long) usageCount.deref();
  }

  public boolean useEnergy(final long units) {
    try {
      return (Boolean) LockingTransaction.runInTransaction(new Callable<Boolean>() {
        @Override
        public Boolean call() {
          if (units > 0 && getUnitsAvailable() >= units) {
            level.set(getUnitsAvailable() - units);
            usageCount.set(getUsage() + 1);
            return true;
          }
          return false;
        }
      });
    } catch (Exception ex) {
      throw new RuntimeException("Failed after large number of attempts", ex);
    }
  }

  private void replenish() {
    try {
      LockingTransaction.runInTransaction(new Callable<Void>() {
        @Override
        public Void call() throws Exception {
          if (getUnitsAvailable() < MAXLEVEL) level.set(getUnitsAvailable() + 1);
          return null;
        }
      });
    } catch (Exception ex) {
      throw new RuntimeException("Failed after large number of attempts", ex);
    }
  }
}
